<?php declare(strict_types=1);

namespace App\Form;

use App\Entity\MenuItem;
use App\Repository\MenuItemRepository;
use Doctrine\ORM\QueryBuilder;
use Override;
use Symfony\Bridge\Doctrine\Form\Type\EntityType;
use Symfony\Bridge\Doctrine\Types\UlidType;
use Symfony\Component\Form\AbstractType;
use Symfony\Component\Form\FormBuilderInterface;
use Symfony\Component\OptionsResolver\OptionsResolver;

final class MenuItemType extends AbstractType
{
    public function buildForm(FormBuilderInterface $builder, array $options): void
    {
        $item = $options['data'];
        assert($item instanceof MenuItem); // Ensure it's of the correct type
        $vendorId = $item->getFoodVendor()?->getId(); // Use safe navigation operator in case FoodVendor is null

        $builder->add('name'); // Basic field
        $builder->add('aliases', EntityType::class, [
                'class' => MenuItem::class,
                'choice_label' => 'name',
                'multiple' => true,
                'expanded' => true,
                'query_builder' => static function (MenuItemRepository $repository) use ($item, $vendorId): QueryBuilder {
                    $qb = $repository->createQueryBuilder('m');

                    // Build the main query with a NOT EXISTS constraint
                    $qb
                        ->where('m.foodVendor = :vendorId')
                        ->andWhere('m.deletedAt IS NULL')
                        ->andWhere('m.id != :id')
                        ->andWhere(
                            $qb->expr()->notIn('m.id',
                                    $repository->createQueryBuilder('m2')
                                        ->select('m2.id')
                                        ->where('m2.aliasOf != m.id') // Reference m.id in the inner query
                                        ->orWhere('m2.aliasOf IS NOT NULL')
                                        ->getDQL() // The subquery's DQL string
                                )
                        )
                        ->orderBy('m.name', 'ASC')
                        ->setParameter('vendorId', $vendorId, UlidType::NAME) // ULID or appropriate type
                        ->setParameter('id', $item->getId(), UlidType::NAME); // ULID or appropriate type

                    return $qb;
                },
            ]);
    }
    #[Override]
    public function configureOptions(OptionsResolver $resolver): void
    {
        $resolver->setDefaults([
            'data_class' => MenuItem::class,
        ]);
    }
}
